/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DistributedJNDIServer.java 6:42:03 PM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.communication.jndi.tribe;

import java.net.InetAddress;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.communication.jndi.AbstractJNDIAgentService;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.NetworkUtil;
import org.objectweb.petals.util.SystemUtil;
import org.objectweb.tribe.common.IpAddress;
import org.objectweb.util.monolog.api.Logger;

/**
 * TRIBe JNDI Server
 *
 * At startup :
 * <ol>
 * <li>The JNDI server will start the socket server and the tribe listener.</li>
 * <li>The JNDI server will search for other members in the group by sending a
 * electMaster request</li>
 * <li>
 * <ul>
 * <li>If the server is alone then it will be its own master (by keeping its
 * master field at null)</li>
 * <li>If there is a master, it will send back its member id and ask for a full
 * update of its data</li>
 * </ul>
 * </li>
 * </ol>
 *
 * At Shutdown :
 * <ul>
 * <li>If the server is not the master, the server does nothing</li>
 * <li>If the server is the master :
 * <ol>
 * <li>The server sends a newMaster request to all the servers</li>
 * <li>The others servers reset their master field</li>
 * <li>The others servers sends a electNewMaster message with their starttime</li>
 * <li>Each server checks if it is the master (the one with the smallest start
 * time)</li>
 * </ol>
 * </li>
 * </ul>
 *
 * @author ddesjardins - eBMWebsourcing
 * @author chamerling - eBMWebsourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.objectweb.petals.communication.jndi.JNDIAgentService.class))
public class TribeJNDIAgentServiceImpl extends AbstractJNDIAgentService {

    /**
     * The distributed JNDI TRIBe
     */
    private TribeServer tribeServer;

    /**
     * The TRIBe multicast Port
     */
    private final static int multicastPort = 2288;

    /**
     * The multicast address used by TRIBe
     */
    private String multicast;

    /**
     * Creates a new instance of {@link TribeJNDIAgentServiceImpl}
     *
     */
    public TribeJNDIAgentServiceImpl() {
        super();
    }

    /*
     * TODO : there is a bug when the logger is defined in the mother abstract
     * class. Move back the method in AbstractJNDIAgentService when fixed.
     */
    /**
     * Logger
     */
    @Monolog(name = "logger")
    protected Logger logger;

    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws Exception {
        log = new LoggingUtil(logger);
        log.start();
        status = Status.NOT_INITIALIZED;
        dynamic = SystemUtil.isDynamicTopology();

        host = SystemUtil.getHost();

        // initialize and start the JNDI server
        initJNDIAgent();
        startJNDIAgent();
        log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() throws Exception {
        log.start();
        try {
            stopJNDIAgent();
        } catch (PetalsException e) {
            log.error(getImplName() + " : Can not stop the JNDI component");
            throw new PetalsException(getImplName()
                + " : Can not stop the JNDI component", e);
        }
        log.end();
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.kernel.registry.AbstractJndiServer#initServer()
     */
    public final void doInit() throws Exception {
        if (dynamic) {
            /*
             * As the NetworkService is not yet started because it needs the
             * JNDI server, which need the TRIBe broadcast. we must calculate
             * the jndi port directly. All others ports are managed by the
             * NetworkService.
             */
            resolveJndiPort();

            multicast = SystemUtil.getTribeMulticastAddress();
            //the port must be re set at each start, otherwise TRIBe doesn't work
            int replyPort = 0;
            String multicastDomain = SystemUtil.getJoramDomain();

            port = Integer.parseInt(SystemUtil.getJndiPort());

            InetAddress multicastAddress = InetAddress.getByName(multicast);
            final IpAddress multicastIP = new IpAddress(multicastAddress,
                multicastPort);
            final InetAddress replyAddress = InetAddress.getByName(host);
            final IpAddress replyIP = new IpAddress(replyAddress, replyPort);

            tribeServer = new TribeServer(host, port, multicastIP, replyIP,
                multicastDomain, log);
            tribeServer.init();
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.kernel.registry.AbstractJndiServer#startServer()
     */
    public void doStart() throws Exception {
        if (dynamic) {
            tribeServer.start();
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.kernel.registry.AbstractJndiServer#stopServer()
     */
    public void doStop() throws Exception {
        if (dynamic) {
            tribeServer.stop();
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.communication.jndi.AbstractJNDIAgentService#getImplName()
     */
    public String getImplName() {
        return "TRIBe";
    }

    /**
     * Resolve the JNDI port; If the server properties file is not defined,
     * retrieve an unused port.
     *
     * @throws PetalsException
     */
    private void resolveJndiPort() throws PetalsException {
        if (SystemUtil.getValue(SystemUtil.JNDI_PORT) == null) {
            int port = Integer.parseInt(SystemUtil.getJndiPort());
            port = NetworkUtil.getNextUnusedPort(host, port);
            SystemUtil.setJndiPort(Integer.toString(port));
        }
    }
}
