/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.task;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.net.URLConnection;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.concurrent.atomic.AtomicLong;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.util.LoggingUtil;

/**
 * This task download entity (component, shared lib, service assembly) archive
 * into the work directory, only if the URL protocol is http: or ftp: <br/> Inputs :
 * archive {@link URI} <br/> Outputs : downloaded archive {@link URI}
 * <br/>
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class DownloadTask extends AbstractFileManipulationTask {

    /**
     * logger wrapper
     */
    protected final LoggingUtil log;

    /**
     * DateFormat used for repository folders
     */
    private final static SimpleDateFormat FOLDER_DATE_FORMAT = new SimpleDateFormat("MM-dd-yyyy-hhmmss");

    /**
     * Unique download identifier, it is used in temporary path name to prevent
     * concurrent downloading.
     */
    private final static AtomicLong DOWNLOAD_ID = new AtomicLong(0L);
    
    public DownloadTask(LoggingUtil log) {
        super();
        this.log = log;
    }

    @SuppressWarnings("unchecked")
    public void execute(HashMap context) throws Exception {

        URI archiveURI = (URI) context.get(TaskContextConstants.ARCHIVE_URI);

        /*
         * Explode Comp
         */
        File unzipRoot = downloadEntityPackage(archiveURI);

        /*
         * Fill context
         */
        context.put(TaskContextConstants.ARCHIVE_URI, unzipRoot.toURI());

    }
    
    /*
     * SimpleDateFormat must be synchronized.
     */
    private static synchronized String getCurrentDateAsString() {
        
        return DownloadTask.FOLDER_DATE_FORMAT.format(new Date(System
                .currentTimeMillis()));
    }

    private File downloadEntityPackage(URI archiveURI) throws PetalsException {
        File outputFile = null;
        try {
            final URLConnection urlConnection = archiveURI.toURL().openConnection();
            // We check if the remote resource exists.
            if ("http".equalsIgnoreCase(archiveURI.getScheme())) {
                final HttpURLConnection httpUrlConnection = (HttpURLConnection)urlConnection;
                final int responseCode = httpUrlConnection.getResponseCode();
                this.log.debug("HTTP Response code: " + responseCode);
                if (responseCode != HttpURLConnection.HTTP_OK) {
                    throw new PetalsException("Unable to download the remote resource " + archiveURI + " [HTTP " + responseCode + "]");
                }
                else {
                    // We check that the resource is a binary file
                    final String contentType = httpUrlConnection.getContentType();
                    if (!"application/zip".equals(contentType)) {
                        throw new PetalsException("The remote resource [" + archiveURI + "] is not a valid JBI archive: " + contentType);
                    }
                }
            }
            
            final InputStream inputStream = urlConnection.getInputStream();
            
            final String pathName = archiveURI.getPath();
            final String fileName = pathName.substring(pathName
                    .lastIndexOf('/') + 1, pathName.lastIndexOf('.'));
            outputFile = new File(getWorkDirectory(), 
                    fileName + "-downloaded-"
                    + DownloadTask.getCurrentDateAsString() + "-"
                    + DownloadTask.DOWNLOAD_ID.getAndIncrement() + ".zip");
            FileOutputStream fileOutputStream = new FileOutputStream(outputFile);
            this.log.debug("Downloading archive " + archiveURI.toString() + " to " + outputFile.toString());
            IOUtils.copy(inputStream, fileOutputStream);
            this.log.debug("Archive downloaded.");
        } catch (MalformedURLException e) {
            // Nothing to do URL already checked;
        } catch (IOException e) {
            new PetalsException("Can't read distant file: " + archiveURI);
        }
        return outputFile;
    }

    public void undo(HashMap context) {
        URI downloadURI = (URI) context.get(TaskContextConstants.ARCHIVE_URI);

        try {
            FileUtils.forceDelete(new File(downloadURI));
        } catch (IOException e) {
            String msg = "Failed to revert a DownloadTask";
            log.error(msg, e);
        }
    }

}
