/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: CreateAndRegisterSULifeCycleTask.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.task.deployment.deploy;

import java.net.URI;
import java.util.HashMap;

import javax.jbi.JBIException;

import org.objectweb.petals.container.lifecycle.ServiceAssemblyLifeCycle;
import org.objectweb.petals.container.lifecycle.ServiceUnitLifeCycle;
import org.ow2.petals.jbi.descriptor.JBIDescriptor;
import org.ow2.petals.jbi.descriptor.ServiceUnit;
import org.objectweb.petals.jbi.management.ManagementException;
import org.objectweb.petals.jbi.management.admin.AdminServiceMBean;
import org.objectweb.petals.jbi.management.task.deployment.DeploymentContextConstants;
import org.objectweb.petals.jbi.management.task.deployment.DeploymentUtils;
import org.objectweb.petals.processor.Task;
import org.objectweb.petals.util.LoggingUtil;

/**
 * This task creates a service unit lifecycle and registers this lifecycle into
 * the sa lifecycle <br/> Inputs : {@link JBIDescriptor} of the SU,
 * {@link ServiceAssemblyLifeCycle}, SU install root {@link URI} <br/>
 *
 * @author ofabre - EBM Websourcing
 *
 */
public class CreateAndRegisterSULifeCycleTask implements Task {

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    /**
     * JBI Container :: JMX Agent Admin
     */
    protected AdminServiceMBean adminService;

    public CreateAndRegisterSULifeCycleTask(LoggingUtil log,
        AdminServiceMBean adminService) {
        super();
        this.log = log;
        this.adminService = adminService;
    }

    @SuppressWarnings("unchecked")
    public void execute(HashMap context) throws Exception {
        URI installationRoot = (URI) context
            .get(DeploymentContextConstants.SU_INSTALL_ROOT);

        ServiceUnit serviceUnit = (ServiceUnit) context
            .get(DeploymentContextConstants.SU_DESCRIPTOR);

        ServiceAssemblyLifeCycle saLifeCycle = (ServiceAssemblyLifeCycle) context
            .get(DeploymentContextConstants.SA_LIFECYCLE);

        /*
         * 2. Create service unit lifecycle
         */
        ServiceUnitLifeCycle suLifeCycle = createNewServiceUnitLifeCycle(
            installationRoot.getPath(), serviceUnit);

        /*
         * 3. Register this service unit lifecycle into the sa lifecycle
         */
        registerSULifeCycle(suLifeCycle, saLifeCycle, serviceUnit);

    }

    /**
     * Creates a new {@link ServiceUnitLifeCycle}
     *
     * @param rootPath
     *            the SU installation root
     * @param serviceUnit
     *            the service unit production element
     * @return a new {@link ServiceUnitLifeCycle} instance
     */
    protected ServiceUnitLifeCycle createNewServiceUnitLifeCycle(
        String rootPath, ServiceUnit serviceUnit) {
        return new ServiceUnitLifeCycle(serviceUnit, adminService, rootPath,
            log);
    }

    /**
     * Register the {@link ServiceUnitLifeCycle} to the
     * {@link ServiceAssemblyLifeCycle}.
     *
     * @param suLifeCycle
     *            the {@link ServiceUnitLifeCycle} to register
     * @param saLifeCycle
     *            the {@link ServiceAssemblyLifeCycle} where
     *            {@link ServiceUnitLifeCycle} will be registered
     * @param serviceUnit
     *            the service unit production element
     * @throws ManagementException
     *             if it fails to register {@link ServiceUnitLifeCycle} to
     *             {@link ServiceAssemblyLifeCycle}
     */
    protected void registerSULifeCycle(ServiceUnitLifeCycle suLifeCycle,
        ServiceAssemblyLifeCycle saLifeCycle, ServiceUnit serviceUnit)
        throws ManagementException {
        String serviceUnitName = DeploymentUtils
                            .getServiceUnitName(serviceUnit);
        try {
            saLifeCycle.registerSUIntoSA(serviceUnitName, suLifeCycle);
        } catch (JBIException e) {
            String msg = "Error while registering the service unit "
                + serviceUnitName
                + " life cycle into the service assembly life cycle"
                + saLifeCycle.getServiceAssembly().getIdentification()
                    .getName() + ".";
            log.error(msg, e);
            throw new ManagementException(msg, e);
        }
    }

    @SuppressWarnings("unchecked")
    public void undo(HashMap context) {
        log.debug("Revert CreateAndRegisterSULifeCycleTask");
        ServiceUnit serviceUnit = (ServiceUnit) context
            .get(DeploymentContextConstants.SU_DESCRIPTOR);

        ServiceAssemblyLifeCycle saLifeCycle = (ServiceAssemblyLifeCycle) context
            .get(DeploymentContextConstants.SA_LIFECYCLE);

        try {
            unregisterSULifeCycle(saLifeCycle, serviceUnit);
        } catch (ManagementException e) {
            String msg = "Failed to revert a CreateAndRegisterSULifeCycleTask";
            log.error(msg, e);
        }

    }

    /**
     * Register the {@link ServiceUnitLifeCycle} to the
     * {@link ServiceAssemblyLifeCycle}.
     *
     * @param saLifeCycle
     *            the {@link ServiceAssemblyLifeCycle} where
     *            {@link ServiceUnitLifeCycle} will be registered
     * @param serviceUnit
     *            the service unit production element of the su to unregister
     * @throws ManagementException
     *             if it fails to register {@link ServiceUnitLifeCycle} to
     *             {@link ServiceAssemblyLifeCycle}
     */
    protected void unregisterSULifeCycle(ServiceAssemblyLifeCycle saLifeCycle,
        ServiceUnit serviceUnit) throws ManagementException {
        saLifeCycle.unregisterSUFromSA(DeploymentUtils
            .getServiceUnitName(serviceUnit));

    }

}
