/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: Transporter.java,v 0.3 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.jbi.messaging.transport;

import org.objectweb.petals.PetalsLifeCycle;
import org.objectweb.petals.jbi.messaging.exchange.MessageExchangeImpl;

/**
 * <p>
 * The <code>TransportProtocol</code> interface is used to send a
 * <code>MessageExchange</code> from a Petals container to another one via a
 * specific protocol.
 * </p>
 * <p>
 * The <code>Dispatcher</code> calls
 * {@link #send(MessageExchangeImpl, String, String, long)} on this
 * <code>TransportProtocol</code> interface to send an exchange. This
 * <code>TransportProtocol</code> sends the <code>MessageExchange</code> to the
 * distant <code>TransportProtocol</code>. When the distant
 * <code>TransportProtocol</code> receives the <code>MessageExchange</code>, it
 * has to call {@link TransportProtocolListener#messageAvailable} or
 * {@link TransportProtocolListener#syncMessageReceived} on its local
 * <code>Dispatcher</code>.
 * </p>
 * <p>
 * The localization of the distant container is in charge of the
 * <code>TransportProtocol</code>.
 * </p>
 * <p>
 * <em>Note that the destination container can be the same than this one.</em>
 * </p>
 * 
 * @version $Rev: 477 $ $Date: 2006-05-29 17:18:07 +0200 (lun., 29 mai 2006) $
 * @since Petals 1.0
 * @author alouis
 * @author garciap
 */
public interface TransportProtocol extends PetalsLifeCycle {

	/**
	 * Name of the network interface, needed to retrieve container information
	 */
	public static final String NETWORK_ITF = "network";

	/**
	 * Name of the jndi interface
	 */
	public static final String JNDI_ITF = "jndi";

	/**
	 * Name of the configuration interface
	 */
	public static final String CONFIGURATION_ITF = "configuration";

	/**
	 * Name of the dispatcher interface, needed to process message reply (see interface documentation).
	 * @see TransportProtoco
	 */
	public static final String DISPATCHER_ITF = "dispatcher";
	
	/**
     * Send the specified exchange to the specified container.
     * 
     * @param exchange
     *            the exchange to send. <b>MUST</b> be non null
     * @param componentName
     *            the name of the distant container. <b>MUST</b> be non null
     * @param containerName
     *            the name of the distant container. <b>MUST</b> be non null
     *            and non empty
     * @param timeToLive
     *            duration of the message life (in ms). 0 for no timeout.
     *            <b>MUST</b> be positive
     * @throws TransportException
     *             If the exchange is invalid, or can not be serialized. If the
     *             distant container can not be found.
     */
    void send(MessageExchangeImpl exchange, final String componentName,
        final String containerName, final long timeToLive)
        throws TransportException;

    /**
     * Inform the <code>Transporter</code> that the specified component is
     * installing. The transporter may need to initialise a device to handle new
     * bound destination.
     * 
     * @param componentName
     *            The unique name of the installed component. <b>MUST</b> be
     *            non null
     * @throws TransportException
     */

    void addDestination(final String componentName) throws TransportException;

    /**
     * Inform the <code>Transporter</code> that the specified component is
     * uninstalling. The <code>Transporter</code> may need to cleanup the
     * device that handles the bound destination.
     * 
     * @param componentName
     *            The unique name of the uninstalled component. <b>MUST</b> be
     *            non null
     * @throws TransportException
     */

    void removeDestination(final String componentName)
        throws TransportException;

    /**
     * Poll a received message for the given component. This is a blocking call.
     * The protocol <b>MUST</b> call the
     * {@link TransportProtocolListener#messageAvailable(String, String) } on
     * the dispatcher before the dispatcher calls this method
     * 
     * @param componentName
     *            The unique name of the component; <b>MUST</b> be non null
     * 
     * @return The received message exchange
     */
    MessageExchangeImpl receive(final String componentName)
        throws TransportException;
   
}
