/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): pierre.garcia@inrialpes.fr  
 */

package org.objectweb.petals.jbi.messaging.transport.util;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.jbi.JBIException;

import org.objectweb.dream.message.ChunkAlreadyExistException;
import org.objectweb.dream.message.ChunkFactoryReference;
import org.objectweb.dream.message.InvalidReference;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageManagerType;
import org.objectweb.dream.protocol.ExportIdentifierChunk;
import org.objectweb.dream.protocol.IPExportIdentifier;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.communication.network.ContainerInformation;
import org.objectweb.petals.communication.network.NetworkService;
import org.objectweb.petals.jbi.messaging.exchange.MessageExchangeImpl;
import org.objectweb.petals.jbi.messaging.transport.TransportException;
import org.objectweb.petals.jbi.messaging.transport.dream.MessageExchangeChunk;
import org.objectweb.petals.util.LoggingUtil;

public class DreamUtil {

	/**
	 * Name of the chunk including destination
	 */
	protected static final String DESTINATION_CHUNK_NAME = "destination";

	/**
	 * Containers ref in the jndi distributed directory
	 */
	protected static final String CONTAINERS_REF = "containers";

	/**
	 * Message Manager. Used here only to manage chunk in a message
	 */
	protected MessageManagerType messageManagerItf;

	/**
	 * Factory used to create Export Identifier Chunk
	 */
	protected ChunkFactoryReference<ExportIdentifierChunk> exportIdChunkFactory;

	/**
	 * Factory used to create, delete, copy... MessageExchangeChunk
	 */
	ChunkFactoryReference<MessageExchangeChunk> chunkFactory;

	/**
	 * Inherited Logger
	 */
	LoggingUtil log;

	/**
	 * Interface used to get remote container informations
	 */
	protected NetworkService networkService;

	/**
	 * Mapping between container id and ipExportChunk
	 */
	protected Map<String, ExportIdentifierChunk> exportIdChunks;

	public DreamUtil(MessageManagerType messageManager, LoggingUtil log,
			NetworkService networkService) {
		this.log = log;
		log.start();
		this.messageManagerItf = messageManager;
		// create the chunkFactory
		this.exportIdChunkFactory = messageManagerItf
				.getChunkFactory(ExportIdentifierChunk.class);
		this.chunkFactory = messageManagerItf
				.getChunkFactory(MessageExchangeChunk.class);
		this.networkService = networkService;

		this.exportIdChunks = new ConcurrentHashMap<String, ExportIdentifierChunk>();

		log.end();
	}

	/**
	 * Add a chunk including the destination of the message
	 * 
	 * @param message
	 * @param host
	 * @param port
	 * @return
	 * @throws TransportException
	 */
	public Message addChunkDestination(Message message, String containerName)
			throws TransportException {
		log.start();
		
		log.debug("add chunk destination for the container " + containerName);

		ExportIdentifierChunk eic = exportIdChunks.get(containerName);

		if (eic == null) {
			eic = createChunkDestination(containerName);
		}
		eic = messageManagerItf.cloneChunk(eic);
		log.debug("Chunk : " + eic);
		messageManagerItf.addOrReplaceChunk(message, DESTINATION_CHUNK_NAME,
				eic);
		
		log.end();
		return message;
	}

	/**
	 * 
	 * 
	 * @param containerName
	 * @return
	 * @throws TransportException
	 */
	protected ExportIdentifierChunk createChunkDestination(String containerName)
			throws TransportException {
		log.start();

		final ContainerInformation containerInformation;
		try {
			containerInformation = this.networkService
					.getContainerInformation(containerName);
		} catch (PetalsException e) {
			throw new TransportException(
					"Cannot get the information for the container "
							+ containerName, e);
		}

		if (containerInformation == null) {
			throw new TransportException(
					"Cannot get the information for the container "
							+ containerName);
		}

		ExportIdentifierChunk eic = null;
		String host = containerInformation.getHost();
		int port = Integer.valueOf(containerInformation.getDreamTCPPort());
		IPExportIdentifier exportIdentifier = new IPExportIdentifier(host, port);
		eic = messageManagerItf.createChunk(exportIdChunkFactory);
		eic.setExportIdentifier(exportIdentifier);

		exportIdChunks.put(containerInformation.getName(), eic);

		log.debug("Export identifier (" + host + ", " + port + ")");
		log.end();
		return eic;
	}

	/**
	 * Transform a jbi message into a message formatted for a specific router.
	 * 
	 * @param jbi
	 *            the message to transform
	 * @param parameter
	 *            map of specific needed objects required by the transporter.
	 * @return the message formatted for the transporter
	 * @throws JBIException
	 *             thrown if an error occurs during serialization
	 */
	public MessageExchangeImpl dream2jbi(Message dreamMessage)
			throws JBIException {
		try {
			final MessageExchangeChunk mec = (MessageExchangeChunk)this.messageManagerItf
					.getChunk(dreamMessage, MessageExchangeChunk.DEFAULT_NAME);
			if (mec == null) {
				throw new JBIException("MessageExchangeChunk not found !");
			} else {
				return mec.getMessageExchange();
			}
		} catch (InvalidReference ir) {
			throw new JBIException("The MessageExchangeChunk "
					+ "Reference is not valid ", ir);
		}
	}

	/**
	 * Transform a transporter format message into a jbi message.
	 * 
	 * @param messageFromTransporter
	 *            message with a specific format related to the transporter
	 * @param stuffs
	 *            map of specific needed.
	 * @return a jbi {@link MessageExchangeImpl message}
	 * @throws TransportException
	 *             thrown if an error occurs during serialization
	 */
	public Message jbi2dream(MessageExchangeImpl jbi) throws TransportException {
		Message dreamMessage = messageManagerItf.createMessage();
		try {
			MessageExchangeChunk mec = messageManagerItf
					.createChunk(chunkFactory);
			mec.setMessageExchange(jbi);
			messageManagerItf.addChunk(dreamMessage,
					MessageExchangeChunk.DEFAULT_NAME, mec);
			return dreamMessage;
		} catch (InvalidReference ir) {
			throw new TransportException("The MessageExchangeChunk "
					+ "Reference is not valid ", ir);
		} catch (ChunkAlreadyExistException caee) {
			throw new TransportException("The Dream Message Already "
					+ "contains a MessageExchangeChunk : this is a dream "
					+ "internal error ", caee);
		}
	}
}
