/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JoramConfigurationBuilder.java 2:04:08 PM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.messaging.transport.util;

import java.io.File;
import java.io.IOException;
import java.net.ConnectException;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.objectweb.joram.client.jms.admin.AdminException;
import org.objectweb.joram.client.jms.admin.AdminModule;
import org.objectweb.petals.communication.network.ContainerInformation;

import fr.dyade.aaa.agent.AgentServer;

/**
 * This class is used to build the Joram configuration file a3servers.xml
 * 
 * @author ddesjardins - eBMWebsourcing
 * @author chamerling - eBMWebsourcing
 */
public class JoramConfigurationBuilder {

    /**
     * Singleton
     */
    private static JoramConfigurationBuilder instance;

    /**
     * The joram control singleton
     */
    private JoramControl joramControl = JoramControl.getInstance();

    /**
     * Creates a new instance of {@link JoramConfigurationBuilder}
     * 
     */
    private JoramConfigurationBuilder() {
    }

    /**
     * Get the instance of the configuration builder
     * 
     * @return
     */
    public static synchronized JoramConfigurationBuilder getInstance() {
        if (instance == null) {
            instance = new JoramConfigurationBuilder();
        }
        return instance;
    }

    /**
     * Build a joram configuration file (a3servers.xml) for a distributed Joram
     * server
     * 
     * @param masterConf
     *            configuration of the others Joram servers
     * @param domain
     *            domain of the server
     * @param id
     *            id of the server
     * @param name
     *            name of the server
     * @param host
     *            host of the server
     * @param username
     *            username to access to the admin module
     * @param password
     *            password to access to the admin module
     * @param domainPort
     *            port of the domain
     * @param tcpPort
     *            port for the tcp connector
     * @param uid
     *            uid of the server
     * @return xml String containing the full configuration
     */
    public String buildFullConfiguration(String masterConf, String domain,
        String id, String name, String host, String username, String password,
        String domainPort, String tcpPort, String uid) {
        StringBuffer xml = new StringBuffer(masterConf);

        // We add only this configuration if its necessary
        int firstIndex = xml.indexOf("id=\"" + id);
        if (firstIndex == -1) {
            xml.insert(xml.indexOf("</config>"), buildSlaveServerConf(domain,
                id, name, host, username, password, domainPort, tcpPort, uid));
        } else {
            // change the default root root user/pwd if neccessary
            int secondIndex = xml.indexOf("root root", firstIndex);
            if (secondIndex > -1) {
                String subString = xml.substring(firstIndex, secondIndex);
                if (subString.indexOf("</server>") == -1) {
                    xml.replace(secondIndex, secondIndex + 9, username + " "
                        + password);
                }
            }
        }

        return xml.toString();
    }

    /**
     * Build the full master configuration from the container information.
     * 
     * @param information
     * @return
     */
    public String buildFullMasterConf(ContainerInformation information) {
        return buildFullMasterConf(information.getJoramDomain(), information
            .getJoramId(), information.getName(), information.getHost(),
            information.getJoramLogin(), information.getJoramPassword(),
            information.getJoramDomainPort(), information.getJoramTCPPort(),
            Long.toString(information.getUid()));
    }

    /**
     * Build a joram configuration file (a3servers.xml) for a single Joram
     * server
     * 
     * @param domain
     *            domain of the server
     * @param id
     *            id of the server
     * @param name
     *            name of the server
     * @param host
     *            host of the server
     * @param username
     *            username to access to the admin module
     * @param password
     *            password to access to the admin module
     * @param domainPort
     *            port of the domain
     * @param tcpPort
     *            port for the tcp connector
     * @param uid
     *            uid of the server
     * @return xml String containing the configuration
     */
    public String buildFullMasterConf(String domain, String id, String name,
        String host, String username, String password, String domainPort,
        String tcpPort, String uid) {
        StringBuffer xml = new StringBuffer();
        xml.append("<?xml version=\"1.0\"?>\n");
        xml.append("<!-- <!DOCTYPE config SYSTEM \"a3config.dtd\">-->\n");
        xml.append("<config>\n");
        xml.append("\t<domain name=\"" + domain + "\"/>\n");
        // xml
        // .append("\t<property name=\"Transaction\"
        // value=\"fr.dyade.aaa.util.NullTransaction\"/>\n");
        xml.append(buildMasterServerConf(domain, id, name, host, username,
            password, domainPort, tcpPort, uid));
        xml.append("</config>\n");

        return xml.toString();
    }

    /**
     * Is the specified server is registered in the Joram global configuration.
     * <b>AdminModule.connect required</b>
     * 
     * @return
     * @throws AdminException
     * @throws ConnectException
     */
    public boolean isServerRegistered(String serverId) throws ConnectException,
        AdminException {
        boolean serverAlreadyExist = false;
        for (Iterator serverIdsIt = AdminModule.getServersIds().iterator(); serverIdsIt
            .hasNext()
            && !serverAlreadyExist;) {
            String aServerId = serverIdsIt.next().toString();
            serverAlreadyExist = (aServerId.equalsIgnoreCase(serverId));
        }
        return serverAlreadyExist;
    }

    /**
     * Retrieve the distributed configuration of Joram platform. Try to connect
     * to each Joram node to get configuration.
     * 
     * @param localContainer
     * @param containers
     * @return
     */
    public String retrieveDistributedConfiguration(
        ContainerInformation localContainer,
        List<ContainerInformation> containers) {
        String conf = null;

        if (containers.size() > 0) {

            boolean updateLocalJoramServer = false;
            for (ContainerInformation information : containers) {
                if (information.getJoramId()
                    .equals(localContainer.getJoramId())) {
                    if (!(information.getJoramDomain().equals(
                        localContainer.getJoramDomain())
                        && information.getJoramDomainPort().equals(
                            localContainer.getJoramDomainPort())
                        && information.getJoramTCPPort().equals(
                            localContainer.getJoramTCPPort())
                        && information.getJoramLogin().equals(
                            localContainer.getJoramLogin())
                        && information.getJoramPassword().equals(
                            localContainer.getJoramPassword()) && information
                        .getUid() == localContainer.getUid())) {
                        updateLocalJoramServer = true;
                        break;
                    }
                }
            }

            for (ContainerInformation information : containers) {
                try {
                    AdminModule.connect(information.getHost(), Integer
                        .parseInt(information.getJoramTCPPort()), information
                        .getJoramLogin(), information.getJoramPassword(), 3);

                    // Retrieve the configuration
                    conf = AdminModule.getConfiguration();

                    // If our server has not been already recorded in
                    // the global conf, add it
                    if (!isServerRegistered(localContainer.getJoramId())) {
                        joramControl.addJoramServer(Integer
                            .parseInt(localContainer.getJoramId()),
                            localContainer.getHost(), localContainer
                                .getJoramDomain(), Integer
                                .parseInt(localContainer.getJoramDomainPort()),
                            localContainer.getName(), localContainer
                                .getJoramTCPPort());
                    } else if (updateLocalJoramServer) {
                        joramControl.removeJoramServer(Integer
                            .parseInt(localContainer.getJoramId()));

                        // Retrieve the configuration again
                        conf = AdminModule.getConfiguration();

                        // Add the new local Joram configuration
                        joramControl.addJoramServer(Integer
                            .parseInt(localContainer.getJoramId()),
                            localContainer.getHost(), localContainer
                                .getJoramDomain(), Integer
                                .parseInt(localContainer.getJoramDomainPort()),
                            localContainer.getName(), localContainer
                                .getJoramTCPPort());
                    }
                    AdminModule.disconnect();
                    break;
                } catch (Exception e) {
                }
            }

            if (conf != null) {
                // Set correct parameters in the Conf if necessary. Add the
                // local configuration to the retrieved one.
                conf = buildFullConfiguration(conf, localContainer
                    .getJoramDomain(), localContainer.getJoramId(),
                    localContainer.getName(), localContainer.getHost(),
                    localContainer.getJoramLogin(), localContainer
                        .getJoramPassword(), localContainer
                        .getJoramDomainPort(),
                    localContainer.getJoramTCPPort(), Long
                        .toString(localContainer.getUid()));
            } else {
                conf = buildFullMasterConf(localContainer);
                // throw new RuntimeException(
                // "Unable to connect to other Petals container, so stop");
            }
        } else {
            conf = buildFullMasterConf(localContainer);
        }
        return conf;
    }

    /**
     * Write the configuration to a3servers.xml file in the joramDir directory.
     * 
     * @param config
     * @param joramDir
     * @throws IOException
     */
    public void writeConfiguration(String config, File joramDir)
        throws IOException {
        FileUtils.writeStringToFile(new File(joramDir,
            AgentServer.DEFAULT_CFG_FILE), config, "UTF-8");
    }

    /**
     * Build the Joram server element of the configuration file (a3servers.xml)
     * 
     * @param domain
     *            domain of the server
     * @param id
     *            id of the server
     * @param name
     *            name of the server
     * @param host
     *            host of the server
     * @param username
     *            username to access to the admin module
     * @param password
     *            password to access to the admin module
     * @param domainPort
     *            port of the domain
     * @param tcpPort
     *            port for the tcp connector
     * @param uid
     *            uid of the server
     * @return xml String representing the element server
     */
    protected String buildMasterServerConf(String domain, String id,
        String name, String host, String username, String password,
        String domainPort, String tcpPort, String uid) {
        StringBuffer xml = new StringBuffer();
        xml.append("\t<server id=\"" + id + "\" name=\"" + name
            + "\" hostname=\"" + host + "\">\n");
        xml.append("\t\t<property name=\"uid\" value=\"" + uid + "\"/>\n");
        xml.append("\t\t<network domain=\"" + domain + "\" port=\""
            + domainPort + "\"/>\n");
        xml
            .append("\t\t<service class=\"org.objectweb.joram.mom.proxies.ConnectionManager\" args=\""
                + username + " " + password + "\"/>\n");
        xml
            .append("\t\t<service class=\"org.objectweb.joram.mom.proxies.tcp.TcpProxyService\" args=\""
                + tcpPort + "\"/>\n");

        // if (JNDI) {
        // xml
        // .append("\t\t<service
        // class=\"fr.dyade.aaa.jndi2.distributed.DistributedJndiServer\"
        // args=\"");
        // xml.append(params);
        // xml.append("\"/>\n");
        // }

        xml.append("\t</server>\n");
        return xml.toString();
    }

    /**
     * Build the Joram server element of the configuration file (a3servers.xml)
     * 
     * @param domain
     *            domain of the server
     * @param id
     *            id of the server
     * @param name
     *            name of the server
     * @param host
     *            host of the server
     * @param username
     *            username to access to the admin module
     * @param password
     *            password to access to the admin module
     * @param domainPort
     *            port of the domain
     * @param tcpPort
     *            port for the tcp connector
     * @param uid
     *            uid of the server
     * @return xml String representing the element server
     */
    protected String buildSlaveServerConf(String domain, String id,
        String name, String host, String username, String password,
        String domainPort, String tcpPort, String uid) {
        StringBuffer xml = new StringBuffer();
        xml.append("\t<server id=\"" + id + "\" name=\"" + name
            + "\" hostname=\"" + host + "\">\n");
        xml.append("\t\t<property name=\"uid\" value=\"" + uid + "\"/>\n");
        xml.append("\t\t<network domain=\"" + domain + "\" port=\""
            + domainPort + "\"/>\n");
        xml
            .append("\t\t<service class=\"org.objectweb.joram.mom.proxies.ConnectionManager\" args=\""
                + username + " " + password + "\"/>\n");
        xml
            .append("\t\t<service class=\"org.objectweb.joram.mom.proxies.tcp.TcpProxyService\" args=\""
                + tcpPort + "\"/>\n");
        // xml
        // .append("\t\t<service
        // class=\"fr.dyade.aaa.jndi2.distributed.DistributedJndiServer\"
        // args=\""
        // + jndiPort + " " + masterId + "\"/>\n");

        xml.append("\t</server>\n");
        return xml.toString();
    }
}
