/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.jbi.messaging.transport.util;

import java.io.File;
import java.io.IOException;
import java.net.ConnectException;
import java.net.UnknownHostException;
import java.util.List;

import javax.jms.ConnectionFactory;

import org.apache.commons.io.FileUtils;
import org.objectweb.joram.client.jms.Destination;
import org.objectweb.joram.client.jms.Queue;
import org.objectweb.joram.client.jms.Topic;
import org.objectweb.joram.client.jms.admin.AdminException;
import org.objectweb.joram.client.jms.admin.AdminModule;
import org.objectweb.joram.client.jms.local.LocalConnectionFactory;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.communication.network.ContainerInformation;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.LoggerFactory;

import fr.dyade.aaa.agent.AgentServer;

/**
 * Utility class to control the Joram agent server.
 * 
 * @author Christophe HAMERLING - eBMWebSourcing
 * 
 */
public class JoramControl {

    /**
     * Instance of the Joram control class
     */
    private static final JoramControl instance = new JoramControl();

    /**
     * The local connection of this Joram agent server
     */
    private final ConnectionFactory connectionFactory;
    
    /**
     * Singleton access
     * 
     * @return
     */
    public static JoramControl getInstance() {
        return JoramControl.instance;
    }

    /**
     * Creates a new instance of {@link JoramControl}
     * 
     */
    private JoramControl() {
        this.connectionFactory = (ConnectionFactory) LocalConnectionFactory.create();

        // FIXME optim multithread
        // connectionFactory.getParameters().multiThreadSync = true;
        // connectionFactory.getParameters().multiThreadSyncDelay = 1;
    }

    /**
     * Clean the workspace
     * 
     */
    public final void cleanAgentServerWorkspace(File joramDir) {
        for (File file : joramDir.listFiles()) {
            if (file.isDirectory()) {
                // We are in a Joram backup directory
                boolean isLock = false;
                for (File joramFile : file.listFiles()) {
                    if ("lock".equals(joramFile.getName())) {
                        // We are looking for a lock file
                        isLock = true;
                        break;
                    }
                }
                if (isLock) {
                    // This Joram server was not shutdown correctly
                    // We need to remove the backup directory
                    try {
                        FileUtils.forceDelete(file);
                    } catch (IOException e) {
                        // Do nothing
                    }
                }
            }
        }
    }

    /**
     * Initialize the agent server
     * 
     * @param agentDir
     * @param id
     * @param factory
     * @throws Exception
     */
    private void doInitAgentServer(File agentDir, short id) throws Exception {
        System.setProperty(AgentServer.CFG_DIR_PROPERTY, agentDir
            .getAbsolutePath());

        AgentServer.init(id, agentDir.getAbsolutePath() + File.separator + id,
            getLoggerFactoryInstance());
    }

    /**
     * Test if the agent server is started
     * 
     * @return
     */
    public final boolean isAgentServerRunning() {
        return 4 == AgentServer.getStatus();
    }

    /**
     * Start the agent server.
     * 
     * @throws Exception
     *             the exception throwed by the joram agent
     */
    public final void startAgentServer() throws Exception {
        AgentServer.start();
    }

    /**
     * Stop the agent server
     * 
     */
    public final void stopAgentServer() {
        fr.dyade.aaa.agent.AgentServer.stop();
    }

    /**
     * Add a Joram server to the Joram platform
     * 
     * @param id
     * @param host
     * @param domain
     * @param domainPort
     * @param name
     * @param tcpPort
     * @throws ConnectException
     * @throws AdminException
     */
    public void addJoramServer(int id, String host, String domain,
        int domainPort, String name, String tcpPort) throws ConnectException,
        AdminException {
        AdminModule
            .addServer(
                id,
                host,
                domain,
                domainPort,
                name,
                new String[] {"org.objectweb.joram.mom.proxies.tcp.TcpProxyService"},
                new String[] {tcpPort});
    }

    /**
     * Remove the Joram server from the Joram platform
     * 
     * @param id
     * @throws AdminException
     * @throws ConnectException
     */
    public void removeJoramServer(int id) throws ConnectException,
        AdminException {
        AdminModule.removeServer(id);
    }

    /**
     * 
     * @param containerInformation
     * @param joramDir
     * @throws PetalsException
     */
    public void initAgentServer(File joramDir, short containerId)
        throws PetalsException {
        if (!isAgentServerRunning()) {
            try {
                cleanAgentServerWorkspace(joramDir);
                doInitAgentServer(joramDir, containerId);
            } catch (Exception e) {
                fr.dyade.aaa.agent.AgentServer.stop(true);
                throw new PetalsException("Joram Agent can not be initialized",
                    e);
            }
        }
    }

    /**
     * Remove a container from joram agent. Connects to the joram module, remove
     * the server and disconnect from the admin module.
     * 
     * @param containerInformation
     * @throws PetalsException
     */
    public void removeContainer(ContainerInformation containerInformation)
        throws PetalsException {
        if (containerInformation == null) {
            throw new IllegalArgumentException(
                "Container information can not be null");
        }

        try {
            synchronized (this) {
                connectToAdminModule(containerInformation);
                AdminModule.removeServer(Integer.parseInt(containerInformation
                    .getJoramId()));
                disconnectFromAdminModule();
            }
        } catch (Exception e) {
            throw new PetalsException(
                "Can not remove container from JORAM agent", e);
        }
    }

    /**
     * Get a local connection factory
     * 
     */
    public ConnectionFactory getConnectionFactory() {
        return this.connectionFactory;
    }

    /**
     * Connect the Admin module to the collocated server
     * 
     * @param containerInformation
     * @throws AdminException
     * @throws UnknownHostException
     * @throws NumberFormatException
     * @throws ConnectException
     * @throws Exception
     * 
     * @see AdminModule#collocatedConnect(String, int, String, String, int,
     *      String)
     */
    public void connectToAdminModule(ContainerInformation containerInformation)
        throws ConnectException, NumberFormatException, UnknownHostException,
        AdminException {
        if (containerInformation == null) {
            throw new IllegalArgumentException(
                "Container information can not be null");
        }

        AdminModule.collocatedConnect(containerInformation.getJoramLogin(),
            containerInformation.getJoramPassword());
    }

    /**
     * Disconnect from the JORAM admin module
     * 
     */
    public void disconnectFromAdminModule() {
        AdminModule.disconnect();
    }

    /**
     * Create a new topic
     * 
     * @param topicName
     *            the name of the topic to be created
     * @see Topic#create()
     */
    public Topic createTopic(String topicName) throws ConnectException,
        AdminException {
        Topic topic = Topic.create(topicName);
        topic.setFreeReading();
        topic.setFreeWriting();
        return topic;
    }

    /**
     * Create a new queue
     * 
     * @param queueName
     *            the name of the queue to be created
     * @see Topic#create()
     */
    public Queue createQueue(String queueName) throws ConnectException,
        AdminException {
        Queue queue = Queue.create(queueName);
        queue.setFreeReading();
        queue.setFreeWriting();
        return queue;
    }

    /**
     * Check if the given destination exists.
     * 
     * @param destination
     *            the destination to check
     */
    public boolean isDestinationExist(Destination destination, String serverId)
        throws ConnectException, AdminException {
        List serverDestinations = AdminModule.getDestinations(Integer
            .parseInt(serverId));

        for (Object serverDestination : serverDestinations) {
            if (((Destination) serverDestination).getName().equals(
                destination.getName())) {
                return true;
            }
        }

        return false;
    }

    /**
     * Get the logger factory to be used by JORAM. It must have been properly
     * initialized at the Petals server start.
     * 
     * @return
     */
    private LoggerFactory getLoggerFactoryInstance() {
        return Monolog.getMonologFactory();
    }
}
