/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.util;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Random;

/**
 * <p>
 * A simple and fast UUID generator.
 * </p>
 * <p>
 * See {@link #getNewID()} for more information on the structure of the
 * generated UUID.
 * </p>
 * 
 * @author chamerling - eBM WebSourcing
 * @author cdeneux - Capgemini Sud
 * 
 */
public final class IDGenerator {

    /**
     * The UUID namespace.
     */
    private final static String UUID_NAMESPACE = "petals:uid:";

    /**
     * Random value size.
     */
    private final static int RANDOM_VALUE_SIZE = 18;

    /**
     * The max size of the UUID string.
     */
    private final static int UUID_SIZE = UUID_NAMESPACE.length()
            + RANDOM_VALUE_SIZE + String.valueOf(System.nanoTime()).length()
			+ Long.toString(Long.MAX_VALUE).length();

    /**
     * The unique instance of the IDGenerator.
     */
    private final static IDGenerator instance;

    /**
     * The unique identifier prefix.
     */
    private final String prefix;

    /**
     * The counter value used as identifier suffix.
     */
    private long value;

    /**
     * The mutex object used to synchronize the counter updates.
     */
    private final Object valueLock;

    // Initialization of the static objects.
    static {
        final Random myRand = new Random();

        final long rand = myRand.nextLong();
        String sid;
        try {
            sid = InetAddress.getLocalHost().toString();
        } catch (UnknownHostException e) {
            sid = Thread.currentThread().getName();
        }
        final StringBuffer sb = new StringBuffer();
        sb.append(sid);
        sb.append(":");
        sb.append(Long.toString(rand));

        byte[] array;
        try {
            final MessageDigest md5 = MessageDigest.getInstance("MD5");
            md5.update(sb.toString().getBytes());
            array = md5.digest();
        } catch (NoSuchAlgorithmException e) {
            // if the MD5 hasher is not reachable, just get the string directly
            array = sb.toString().getBytes();
        }

        final StringBuffer sb2 = new StringBuffer();
        for (int j = 0; j < array.length; ++j) {
            final int b = array[j] & 0xFF;
            sb2.append(Integer.toHexString(b));
        }
        int begin = myRand.nextInt();
        if (begin < 0)
            begin = begin * -1;
        begin = begin % 8;

        instance = new IDGenerator(UUID_NAMESPACE
                + sb2.toString().substring(begin, begin + RANDOM_VALUE_SIZE)
                        .toUpperCase());
    }

    /**
     * No public constructor because IDGenerator is a singleton.
     * 
     * @param prefix
     *            The prefix of the unique identifier.
     */
    private IDGenerator(final String prefix) {
        this.value = 0L;
        this.valueLock = new Object();
        this.prefix = prefix;
    }

    /**
     * Get the unique IDGenerator instance.
     * 
     * @return The unique IDGenerator instance.
     */
    public static IDGenerator getInstance() {
        return IDGenerator.instance;
    }

	/**
	 * <p>
	 * Generates an unique identifier. An identifier value has never been
	 * returned (existed) before its calculation.
	 * </p>
	 * 
	 * <p>
	 * The identifier is composed of three part:
	 * <ol>
	 * <li>an UUID used as prefix. This prefix is always different for each
	 * PEtALS launch,</li>
	 * <li>a nano second time</li>
	 * <li>a counter incremeted for each value, starting from 0, ending to
	 * {@link Long#MAX_VALUE}. When the counter reach {@link Long#MAX_VALUE}, it
	 * is reinitialized to 0.</li>
	 * </ol>
	 * </p>
	 * <p>
	 * The returned identifier can be considered as an UUID because :
	 * <ul>
	 * <li>the UUID part determines an unique value for each PEtALS instance in
	 * the world,</li>
	 * <li>the couple made up nano time and counter determines an unique value
	 * for each message of a PEtALS instance, except if we are able to generate
	 * {@link Long#MAX_VALUE} messages during 1 nano-second.</li>
	 * </ul>
	 * </p>
	 * 
	 * @return A unique identifier
	 */
    public String getNewID() {
        final long tmp;
        synchronized (this.valueLock) {
            if ((tmp = ++this.value) >= Long.MAX_VALUE) {
                this.value = 0;
            }
        }
        // We could use a StringBuilder because it is used only into the same
        // thread
        return new StringBuilder(UUID_SIZE).append(this.prefix).append(
                System.nanoTime()).append(tmp).toString();
    }
}