/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: WSDLUtil.java 14:50:31 ddesjardins $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.util;

import java.util.ArrayList;
import java.util.List;

import javax.xml.namespace.QName;

import org.objectweb.petals.PetalsException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * This is used to extract the interfaces name of a WSDL file
 *
 * @author ddesjardins - eBMWebsourcing
 */
public final class WSDLUtil {

    private static final String WSDL1_1_NAMESPACE = "http://schemas.xmlsoap.org/wsdl/";

    private static final String WSDL2_0_NAMESPACE = "http://www.w3.org/2006/01/wsdl";

    private WSDLUtil() {
        // avoid to be built
    }

    /**
     * Retrieve the interfaces matching the given service from the WSDL document
     * provided.
     *
     * @param document Not null, and root element not null.
     * @param serviceName
     * @return The list of the interfaces retrieved, non {@code null} or empty.
     * @throws PetalsException
     *             Failed the retrieve interfaces
     */
    public static List<QName> getInterfacesForService(final Document document, final QName serviceName)
            throws PetalsException {
        final List<QName> interfaces = new ArrayList<QName>();

        if (document == null) {
            throw new PetalsException("Document is null.");
        }
        final Element rootElement = document.getDocumentElement();
        if (rootElement == null) {
            throw new PetalsException("Root element is null.");
        }
        // Retrieve the target namespace
        final String targetNameSpace = rootElement.getAttribute("targetNamespace");

        if (targetNameSpace != null && targetNameSpace.equals(serviceName.getNamespaceURI())) {

            String wsdlNamespace;
            if (WSDL1_1_NAMESPACE.equals(rootElement.getNamespaceURI())) {
                wsdlNamespace = WSDL1_1_NAMESPACE;
            } else if (WSDL2_0_NAMESPACE.equals(rootElement.getNamespaceURI())) {
                wsdlNamespace = WSDL2_0_NAMESPACE;
            } else {
                throw new PetalsException("Incorrect wsdl namespace or WSDL Unsupported ");
            }

            // Retrieve the services
            final NodeList services = rootElement.getElementsByTagNameNS(wsdlNamespace, "service");
            if (services.getLength() == 0) {
                throw new PetalsException("No service element defined in the WSDL document");
            }
            
            final List<String> bindingNames = new ArrayList<String>();
            if (wsdlNamespace.contains(WSDL1_1_NAMESPACE)) {
                
                /*
                 * Search the interface in a WSDL 1.1
                 */
                boolean serviceFound = false;
                for (int i = 0; i < services.getLength(); i++) {
                    final Element service = (Element) services.item(i);
                    if (service.getAttribute("name").equals(serviceName.getLocalPart())) {
                        serviceFound = true;
                        final NodeList ports = service.getElementsByTagNameNS(wsdlNamespace, "port");
                        for (int j = 0; j < ports.getLength(); j++) {
                            final Element port = (Element) ports.item(j);
                            String name = port.getAttribute("binding");
                            if (name.indexOf(':') > -1) {
                                name = name.substring(name.indexOf(':') + 1);
                            }
                            bindingNames.add(name);
                        }
                        break;
                    }
                }

                if (!serviceFound) {
                    throw new PetalsException("No service element named '"
                            + serviceName.getLocalPart() + "' found in the WSDL document");
                }

                if (bindingNames.isEmpty()) {
                    throw new PetalsException("No binding names defined in the service '"
                            + serviceName + "' of the WSDL document");
                }

                final List<String> portNames = new ArrayList<String>();
                // Retrieve the bindings
                final NodeList bindings = rootElement.getElementsByTagNameNS(wsdlNamespace, "binding");
                for (int i = 0; i < bindings.getLength(); i++) {
                    final Element binding = (Element) bindings.item(i);
                    final String name = binding.getAttribute("name");
                    if (bindingNames.contains(name)) {
                        String type = binding.getAttribute("type");
                        if (type.indexOf(':') > -1) {
                            type = type.substring(type.indexOf(':') + 1);
                        }
                        portNames.add(type);
                    }
                }

                if (portNames.isEmpty()) {
                    throw new PetalsException(
                            "No portType defined in the bindings elements of the WSDL document");
                }

                // Retrieve the port types
                final NodeList portTypes = rootElement.getElementsByTagNameNS(wsdlNamespace, "portType");
                for (int i = 0; i < portTypes.getLength(); i++) {
                    final Element portType = (Element) portTypes.item(i);
                    final String name = portType.getAttribute("name");
                    
                    if (portNames.contains(name)) {
                        interfaces.add(new QName(targetNameSpace, name));
                    }
                }

                if (interfaces.isEmpty()) {
                    throw new PetalsException(
                            "No portType element matches the portType names defined in the bindings element of the WSDL document");
                }
            } else if (wsdlNamespace.contains(WSDL2_0_NAMESPACE)) {
                
                
                /*
                 * Search the interface in a WSDL 2.0
                 */
                boolean serviceFound = false;
                for (int i = 0; i < services.getLength(); i++) {
                    final Element service = (Element) services.item(i);
                    if (service.getAttribute("name").equals(serviceName.getLocalPart())) {
                        serviceFound = true;
                        String itf = QName.valueOf(service.getAttribute("interface"))
                                .getLocalPart();
                        final int prefixIndex = itf.indexOf(':');
                        if (prefixIndex > 0) {
                            itf = itf.substring(prefixIndex + 1, itf.length());
                        }
                        interfaces.add(new QName(targetNameSpace, itf));
                    }
                    break;
                }

                if (!serviceFound) {
                    throw new PetalsException("No service element named '"
                            + serviceName.getLocalPart() + "' found in the WSDL document");
                }

                if (interfaces.isEmpty()) {
                    throw new PetalsException(
                            "No interface name defined in the service element '"
                                    + serviceName.getLocalPart() + "' of the WSDL document");
                }
            } else {
                throw new PetalsException("The namespace '" + wsdlNamespace
                        + "' is not a WSDL namespace");
            }

        } else {
            throw new PetalsException("Failed to retrieve the WSDL namespace in the WSDL document");
        }

        return interfaces;
    }
}
