/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2009 OW2, http://www.ow2.org/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: $
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.extensions.transporter.nio.client;

import org.apache.commons.pool.impl.GenericKeyedObjectPool;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.petals.communication.network.NetworkService;
import org.objectweb.petals.jbi.messaging.transport.TransportProtocol;
import org.objectweb.petals.kernel.configuration.LocalConfiguration;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.extensions.transporter.nio.Parameters;
import org.ow2.petals.extensions.transporter.nio.selector.NioClient;
import org.ow2.petals.extensions.transporter.nio.selector.NioSelector;

/**
 * This class is only a Fractal Wrapper of the Apache generic keyed object pool.
 * 
 * @author Christophe DENEUX (Capgemini Sud)
 * 
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = NioClientObjectPool.class))
public class NioClientObjectPoolImpl implements NioClientObjectPool {

    /**
     * Fractal Logger
     */
    @Monolog(name = "logger")
    protected Logger logger;

    /**
     * Interface used to access configuration
     */
    @Requires(name = TransportProtocol.CONFIGURATION_ITF, signature = LocalConfiguration.class)
    protected LocalConfiguration configurationService;

    /**
     * Interface used to get host and port of message recipient
     */
    @Requires(name = TransportProtocol.NETWORK_ITF, signature = NetworkService.class)
    protected NetworkService networkService;

    /**
     * <p>
     * The socket selector Fractal component to use to process network IO.
     * </p>
     */
    @Requires(name = NioSelector.NIO_SELECTOR_ITF, signature = NioSelector.class)
    protected NioSelector selectorAgent;

    /**
     * All connections to remote containers are pooled into a keyed-pool. The
     * key is the name of the remote container.
     */
    private GenericKeyedObjectPool clientConnections;

    /**
     * Logger Wrapper
     */
    protected LoggingUtil log;

    public NioClient borrowObject(final String targetContainerName) throws Exception {
        return (NioClient) this.clientConnections.borrowObject(targetContainerName);
    }

    public void returnObject(final String targetContainerName, final NioClient clientConnection)
            throws Exception {
        this.clientConnections.returnObject(targetContainerName, clientConnection);

    }

    @LifeCycle(on = LifeCycleType.START)
    public void start() throws IllegalLifeCycleException, NoSuchInterfaceException {
        this.log = new LoggingUtil(logger);
        this.log.start();

        // Connection pool initialization
        final GenericKeyedObjectPool.Config config = new GenericKeyedObjectPool.Config();
        config.minIdle = 0;
        config.maxActive = Integer.parseInt(this.configurationService.getValue(
				Parameters.PROP_NAME_MAX_ACTIVE, Parameters.DEFAULT_MAX_ACTIVE));
        config.testOnBorrow = true;
        config.minEvictableIdleTimeMillis = Long.parseLong(this.configurationService.getValue(
        		Parameters.PROP_NAME_MIN_IDLE_TIME, Parameters.DEFAULT_MIN_IDLE_TIME));
        config.timeBetweenEvictionRunsMillis = Long.parseLong(this.configurationService.getValue(
        		Parameters.PROP_NAME_EVICT_RUN_TIME, Parameters.DEFAULT_EVICT_RUN_TIME));

        this.clientConnections = new GenericKeyedObjectPool(
				new NioClientAgentPoolObjectFactory(
						this.networkService,
						this.selectorAgent,
						Integer.parseInt(this.configurationService.getValue(
								Parameters.PROP_NAME_CONNECTION_TIMEOUT,
								Parameters.DEFAULT_CONNECTION_TIMEOUT)),
						this.logger),
        		config);
    }

    /**
     * Method called when the fractal component is stopped
     * 
     * @throws Exception
     */
    @LifeCycle(on = LifeCycleType.STOP)
    public void stop() throws Exception {
        this.log.start();

        // Close the connection pool and disconnect all clients
        this.clientConnections.close();

        this.log.end();
    }

}
